{-# LINE 29 "Bipartiteness.lhs" #-}
  module Algos.Graphs.Bipartiteness ( isBipartite
                                    ) where
 
  import Algos.Graphs.BFS (bfs)
  import Algos.Graphs.Forest (Forest(..), Tree(..), Subtree(..), toUpTree)
  import Algos.Graphs.Graph (AdjList(..), V(..), E(..), GraphStructure(..))
  import Data.Array (array, (!))
  import Data.List (find)
  import Data.Maybe (fromJust)
{-# LINE 62 "Bipartiteness.lhs" #-}
  isBipartite :: AdjList v vl el -> Either [E] ([V],[V])
{-# LINE 69 "Bipartiteness.lhs" #-}
  isBipartite g = maybe (Right bip) (Left . oddCycle f) odde
    where  f     =  bfs g
           bip   =  bipartition f
           odde  =  oddEdge bip g
{-# LINE 84 "Bipartiteness.lhs" #-}
  bipartition    ::  Forest V E -> ([V], [V])
  bipartition f  =   (concat us, concat ws)
    where  (us, ws)         =  unzip $ map bip (trees f)
           bip (Tree v ts)  =  (v : concat ws', concat us')
             where  (us', ws') = unzip $ map (\(Subtree _ t) -> bip t) ts
{-# LINE 111 "Bipartiteness.lhs" #-}
  data Side = U | W deriving Eq
 
  oddEdge             ::  ([V], [V]) -> AdjList v vl el -> Maybe E
  oddEdge (us, ws) g  =   find (\(E _ t h) -> bipa ! (vIx t) == bipa ! (vIx h)) (gEdges (adjGraph g))
    where bipa  =  array (1, gNumVertices (adjGraph g)) $ 
                   (zip (map vIx us) (repeat U)) ++ (zip (map vIx ws) (repeat W))
{-# LINE 135 "Bipartiteness.lhs" #-}
  oddCycle      ::  Forest V E -> E -> [E]
  oddCycle f e  =   reverse tp' ++ [e] ++ map flipEdge hp'
    where uf                =  toUpTree f
          tp                =  upPath (eTail e)
          hp                =  upPath (eHead e)
          upPath v          =  pathEdges $ fromJust (find (\(w, _) -> vIx v == vIx w) uf)
          pathEdges (_, p)  =  map fst p
          pairs             =  zip tp hp
          (tp',hp')         =  unzip (takeWhile (\(e,f) -> eIx e /= eIx f) pairs)
          flipEdge e        =  e { eTail = eHead e, eHead = eTail e }
